import os

MYSQL_DATABASE = os.environ["MYSQL_DATABASE"]
assert MYSQL_DATABASE

MYSQL_USER = os.environ["MYSQL_USER"]
assert MYSQL_USER

MYSQL_PASSWORD = os.environ["MYSQL_PASSWORD"]
assert MYSQL_PASSWORD

MYSQL_PORT = os.environ["MYSQL_PORT"]
assert MYSQL_PORT
try:
    MYSQL_PORT = int(MYSQL_PORT)
except ValueError:
    raise RuntimeError("Failed to parse mysql port from environment")

AOI_OPERATOR_INITIAL_PASSWORD = os.environ["AOI_OPERATOR_INITIAL_PASSWORD"]
assert AOI_OPERATOR_INITIAL_PASSWORD

AOI_ADMINISTRATOR_INITIAL_PASSWORD = os.environ["AOI_ADMINISTRATOR_INITIAL_PASSWORD"]
assert AOI_ADMINISTRATOR_INITIAL_PASSWORD

STORAGE_SECRET = os.environ["STORAGE_SECRET"]
assert STORAGE_SECRET and len(STORAGE_SECRET) > 10

MINIO_PORT = os.environ["MINIO_PORT"]
assert MINIO_PORT

MINIO_ROOT_USER = os.environ["MINIO_ROOT_USER"]
assert MINIO_ROOT_USER

MINIO_ROOT_PASSWORD = os.environ["MINIO_ROOT_PASSWORD"]
assert MINIO_ROOT_PASSWORD
